@@ -59,7 +59,7 @@
 typedef unsigned long	ulong;
 
 static void	raise_strict(VALUE obj);
-static void	dump_val(VALUE obj, int depth, Out out);
+static void	dump_val(VALUE obj, int depth, Out out, int argc, VALUE *argv);
 static void	dump_nil(Out out);
 static void	dump_true(Out out);
 static void	dump_false(Out out);
@@ -87,7 +87,7 @@ static void	dump_data_strict(VALUE obj, Out out);
 static void	dump_data_null(VALUE obj, Out out);
 static void	dump_data_comp(VALUE obj, int depth, Out out);
 static void	dump_data_obj(VALUE obj, int depth, Out out);
-static void	dump_obj_comp(VALUE obj, int depth, Out out);
+static void	dump_obj_comp(VALUE obj, int depth, Out out, int argc, VALUE *argv);
 static void	dump_obj_obj(VALUE obj, int depth, Out out);
 static void	dump_struct_comp(VALUE obj, int depth, Out out);
 static void	dump_struct_obj(VALUE obj, int depth, Out out);
@@ -441,13 +441,17 @@ dump_bignum(VALUE obj, Out out) {
     *out->cur = '\0';
 }
 
+static const char	inf_val[] = INF_VAL;
+static const char	ninf_val[] = NINF_VAL;
+static const char	nan_val[] = NAN_VAL;
+
 // Removed dependencies on math due to problems with CentOS 5.4.
 static void
 dump_float(VALUE obj, Out out) {
     char	buf[64];
     char	*b;
     double	d = rb_num2dbl(obj);
-    int		cnt;
+    int		cnt = 0;
 
     if (0.0 == d) {
 	b = buf;
@@ -457,43 +461,106 @@ dump_float(VALUE obj, Out out) {
 	*b++ = '\0';
 	cnt = 3;
     } else if (OJ_INFINITY == d) {
-	switch (out->opts->mode) {
-	case StrictMode:
-	    raise_strict(obj);
-	case NullMode:
-	    strcpy(buf, "null");
-	    cnt = 4;
-	    break;
-	default:
-	    strcpy(buf, "Infinity");
-	    cnt = 8;
-	    break;
+	if (ObjectMode == out->opts->mode) {
+	    strcpy(buf, inf_val);
+	    cnt = sizeof(inf_val) - 1;
+	} else {
+	    NanDump	nd = out->opts->dump_opts.nan_dump;
+	    
+	    if (AutoNan == nd) {
+		switch (out->opts->mode) {
+		case CompatMode:	nd = WordNan;	break;
+		case StrictMode:	nd = RaiseNan;	break;
+		case NullMode:		nd = NullNan;	break;
+		default:				break;
+		}
+	    }
+	    switch (nd) {
+	    case RaiseNan:
+		raise_strict(obj);
+		break;
+	    case WordNan:
+		strcpy(buf, "Infinity");
+		cnt = 8;
+		break;
+	    case NullNan:
+		strcpy(buf, "null");
+		cnt = 4;
+		break;
+	    case HugeNan:
+	    default:
+		strcpy(buf, inf_val);
+		cnt = sizeof(inf_val) - 1;
+		break;
+	    }
 	}
     } else if (-OJ_INFINITY == d) {
-	switch (out->opts->mode) {
-	case StrictMode:
-	    raise_strict(obj);
-	case NullMode:
-	    strcpy(buf, "null");
-	    cnt = 4;
-	    break;
-	default:
-	    strcpy(buf, "-Infinity");
-	    cnt = 9;
-	    break;
+	if (ObjectMode == out->opts->mode) {
+	    strcpy(buf, ninf_val);
+	    cnt = sizeof(ninf_val) - 1;
+	} else {
+	    NanDump	nd = out->opts->dump_opts.nan_dump;
+	    
+	    if (AutoNan == nd) {
+		switch (out->opts->mode) {
+		case CompatMode:	nd = WordNan;	break;
+		case StrictMode:	nd = RaiseNan;	break;
+		case NullMode:		nd = NullNan;	break;
+		default:				break;
+		}
+	    }
+	    switch (nd) {
+	    case RaiseNan:
+		raise_strict(obj);
+		break;
+	    case WordNan:
+		strcpy(buf, "-Infinity");
+		cnt = 9;
+		break;
+	    case NullNan:
+		strcpy(buf, "null");
+		cnt = 4;
+		break;
+	    case HugeNan:
+	    default:
+		strcpy(buf, ninf_val);
+		cnt = sizeof(ninf_val) - 1;
+		break;
+	    }
 	}
     } else if (isnan(d)) {
-	switch (out->opts->mode) {
-	case StrictMode:
-	    raise_strict(obj);
-	case NullMode:
-	    strcpy(buf, "null");
-	    cnt = 4;
-	    break;
-	default:
-	    strcpy(buf, "NaN");
-	    cnt = 3;
-	    break;
+	if (ObjectMode == out->opts->mode) {
+	    strcpy(buf, nan_val);
+	    cnt = sizeof(nan_val) - 1;
+	} else {
+	    NanDump	nd = out->opts->dump_opts.nan_dump;
+	    
+	    if (AutoNan == nd) {
+		switch (out->opts->mode) {
+		case CompatMode:	nd = WordNan;	break;
+		case StrictMode:	nd = RaiseNan;	break;
+		case NullMode:		nd = NullNan;	break;
+		default:				break;
+		}
+	    }
+	    switch (nd) {
+	    case RaiseNan:
+		raise_strict(obj);
+		break;
+	    case WordNan:
+		strcpy(buf, "NaN");
+		cnt = 3;
+		break;
+	    case NullNan:
+		strcpy(buf, "null");
+		cnt = 4;
+		break;
+	    case HugeNan:
+	    default:
+		strcpy(buf, nan_val);
+		cnt = sizeof(nan_val) - 1;
+		break;
+	    }
 	}
     } else if (d == (double)(long long int)d) {
 	cnt = snprintf(buf, sizeof(buf), "%.1f", d);
@@ -703,32 +770,32 @@ dump_array(VALUE a, VALUE clas, int depth, Out out) {
 	if (0 < id) {
 	    *out->cur++ = ',';
 	}
-	if (0 == out->opts->dump_opts) {
-	    size = d2 * out->indent + 2;
+	if (out->opts->dump_opts.use) {
+	    size = d2 * out->opts->dump_opts.indent_size + out->opts->dump_opts.array_size + 1;
 	} else {
-	    size = d2 * out->opts->dump_opts->indent_size + out->opts->dump_opts->array_size + 1;
+	    size = d2 * out->indent + 2;
 	}
 	cnt--;
 	for (i = 0; i <= cnt; i++) {
 	    if (out->end - out->cur <= (long)size) {
 		grow(out, size);
 	    }
-	    if (0 == out->opts->dump_opts) {
-		fill_indent(out, d2);
-	    } else {
-		if (0 < out->opts->dump_opts->array_size) {
-		    strcpy(out->cur, out->opts->dump_opts->array_nl);
-		    out->cur += out->opts->dump_opts->array_size;
+	    if (out->opts->dump_opts.use) {
+		if (0 < out->opts->dump_opts.array_size) {
+		    strcpy(out->cur, out->opts->dump_opts.array_nl);
+		    out->cur += out->opts->dump_opts.array_size;
 		}
-		if (0 < out->opts->dump_opts->indent_size) {
+		if (0 < out->opts->dump_opts.indent_size) {
 		    int	i;
 		    for (i = d2; 0 < i; i--) {
-			strcpy(out->cur, out->opts->dump_opts->indent);
-			out->cur += out->opts->dump_opts->indent_size;
+			strcpy(out->cur, out->opts->dump_opts.indent_str);
+			out->cur += out->opts->dump_opts.indent_size;
 		    }
 		}
+	    } else {
+		fill_indent(out, d2);
 	    }
-	    dump_val(rb_ary_entry(a, i), d2, out);
+	    dump_val(rb_ary_entry(a, i), d2, out, 0, 0);
 	    if (i < cnt) {
 		*out->cur++ = ',';
 	    }
@@ -737,22 +804,22 @@ dump_array(VALUE a, VALUE clas, int depth, Out out) {
 	if (out->end - out->cur <= (long)size) {
 	    grow(out, size);
 	}
-	if (0 == out->opts->dump_opts) {
-	    fill_indent(out, depth);
-	} else {
+	if (out->opts->dump_opts.use) {
 	    //printf("*** d2: %u  indent: %u '%s'\n", d2, out->opts->dump_opts->indent_size, out->opts->dump_opts->indent);
-	    if (0 < out->opts->dump_opts->array_size) {
-		strcpy(out->cur, out->opts->dump_opts->array_nl);
-		out->cur += out->opts->dump_opts->array_size;
+	    if (0 < out->opts->dump_opts.array_size) {
+		strcpy(out->cur, out->opts->dump_opts.array_nl);
+		out->cur += out->opts->dump_opts.array_size;
 	    }
-	    if (0 < out->opts->dump_opts->indent_size) {
+	    if (0 < out->opts->dump_opts.indent_size) {
 		int	i;
 
 		for (i = depth; 0 < i; i--) {
-		    strcpy(out->cur, out->opts->dump_opts->indent);
-		    out->cur += out->opts->dump_opts->indent_size;
+		    strcpy(out->cur, out->opts->dump_opts.indent_str);
+		    out->cur += out->opts->dump_opts.indent_size;
 		}
 	    }
+	} else {
+	    fill_indent(out, depth);
 	}
 	*out->cur++ = ']';
     }
@@ -767,7 +834,10 @@ hash_cb_strict(VALUE key, VALUE value, Out out) {
     if (rb_type(key) != T_STRING) {
 	rb_raise(rb_eTypeError, "In :strict mode all Hash keys must be Strings, not %s.\n", rb_class2name(rb_obj_class(key)));
     }
-    if (0 == out->opts->dump_opts) {
+    if (out->omit_nil && Qnil == value) {
+	return ST_CONTINUE;
+    }
+    if (!out->opts->dump_opts.use) {
 	size = depth * out->indent + 1;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
@@ -776,37 +846,37 @@ hash_cb_strict(VALUE key, VALUE value, Out out) {
 	dump_str_comp(key, out);
 	*out->cur++ = ':';
     } else {
-	size = depth * out->opts->dump_opts->indent_size + out->opts->dump_opts->hash_size + 1;
+	size = depth * out->opts->dump_opts.indent_size + out->opts->dump_opts.hash_size + 1;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
 	}
-	if (0 < out->opts->dump_opts->hash_size) {
-	    strcpy(out->cur, out->opts->dump_opts->hash_nl);
-	    out->cur += out->opts->dump_opts->hash_size;
+	if (0 < out->opts->dump_opts.hash_size) {
+	    strcpy(out->cur, out->opts->dump_opts.hash_nl);
+	    out->cur += out->opts->dump_opts.hash_size;
 	}
-	if (0 < out->opts->dump_opts->indent_size) {
+	if (0 < out->opts->dump_opts.indent_size) {
 	    int	i;
 	    for (i = depth; 0 < i; i--) {
-		strcpy(out->cur, out->opts->dump_opts->indent);
-		out->cur += out->opts->dump_opts->indent_size;
+		strcpy(out->cur, out->opts->dump_opts.indent_str);
+		out->cur += out->opts->dump_opts.indent_size;
 	    }
 	}
 	dump_str_comp(key, out);
-	size = out->opts->dump_opts->before_size + out->opts->dump_opts->after_size + 2;
+	size = out->opts->dump_opts.before_size + out->opts->dump_opts.after_size + 2;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
 	}
-	if (0 < out->opts->dump_opts->before_size) {
-	    strcpy(out->cur, out->opts->dump_opts->before_sep);
-	    out->cur += out->opts->dump_opts->before_size;
+	if (0 < out->opts->dump_opts.before_size) {
+	    strcpy(out->cur, out->opts->dump_opts.before_sep);
+	    out->cur += out->opts->dump_opts.before_size;
 	}
 	*out->cur++ = ':';
-	if (0 < out->opts->dump_opts->after_size) {
-	    strcpy(out->cur, out->opts->dump_opts->after_sep);
-	    out->cur += out->opts->dump_opts->after_size;
+	if (0 < out->opts->dump_opts.after_size) {
+	    strcpy(out->cur, out->opts->dump_opts.after_sep);
+	    out->cur += out->opts->dump_opts.after_size;
 	}
     }
-    dump_val(value, depth, out);
+    dump_val(value, depth, out, 0, 0);
     out->depth = depth;
     *out->cur++ = ',';
 
@@ -818,26 +888,29 @@ hash_cb_compat(VALUE key, VALUE value, Out out) {
     int		depth = out->depth;
     long	size;
 
-    if (0 == out->opts->dump_opts) {
+    if (out->omit_nil && Qnil == value) {
+	return ST_CONTINUE;
+    }
+    if (!out->opts->dump_opts.use) {
 	size = depth * out->indent + 1;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
 	}
 	fill_indent(out, depth);
     } else {
-	size = depth * out->opts->dump_opts->indent_size + out->opts->dump_opts->hash_size + 1;
+	size = depth * out->opts->dump_opts.indent_size + out->opts->dump_opts.hash_size + 1;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
 	}
-	if (0 < out->opts->dump_opts->hash_size) {
-	    strcpy(out->cur, out->opts->dump_opts->hash_nl);
-	    out->cur += out->opts->dump_opts->hash_size;
+	if (0 < out->opts->dump_opts.hash_size) {
+	    strcpy(out->cur, out->opts->dump_opts.hash_nl);
+	    out->cur += out->opts->dump_opts.hash_size;
 	}
-	if (0 < out->opts->dump_opts->indent_size) {
+	if (0 < out->opts->dump_opts.indent_size) {
 	    int	i;
 	    for (i = depth; 0 < i; i--) {
-		strcpy(out->cur, out->opts->dump_opts->indent);
-		out->cur += out->opts->dump_opts->indent_size;
+		strcpy(out->cur, out->opts->dump_opts.indent_str);
+		out->cur += out->opts->dump_opts.indent_size;
 	    }
 	}
     }
@@ -853,24 +926,24 @@ hash_cb_compat(VALUE key, VALUE value, Out out) {
 	dump_str_comp(rb_funcall(key, oj_to_s_id, 0), out);
 	break;
     }
-    if (0 == out->opts->dump_opts) {
+    if (!out->opts->dump_opts.use) {
 	*out->cur++ = ':';
     } else {
-	size = out->opts->dump_opts->before_size + out->opts->dump_opts->after_size + 2;
+	size = out->opts->dump_opts.before_size + out->opts->dump_opts.after_size + 2;
 	if (out->end - out->cur <= size) {
 	    grow(out, size);
 	}
-	if (0 < out->opts->dump_opts->before_size) {
-	    strcpy(out->cur, out->opts->dump_opts->before_sep);
-	    out->cur += out->opts->dump_opts->before_size;
+	if (0 < out->opts->dump_opts.before_size) {
+	    strcpy(out->cur, out->opts->dump_opts.before_sep);
+	    out->cur += out->opts->dump_opts.before_size;
 	}
 	*out->cur++ = ':';
-	if (0 < out->opts->dump_opts->after_size) {
-	    strcpy(out->cur, out->opts->dump_opts->after_sep);
-	    out->cur += out->opts->dump_opts->after_size;
+	if (0 < out->opts->dump_opts.after_size) {
+	    strcpy(out->cur, out->opts->dump_opts.after_sep);
+	    out->cur += out->opts->dump_opts.after_size;
 	}
     }
-    dump_val(value, depth, out);
+    dump_val(value, depth, out, 0, 0);
     out->depth = depth;
     *out->cur++ = ',';
 
@@ -882,6 +955,9 @@ hash_cb_object(VALUE key, VALUE value, Out out) {
     int		depth = out->depth;
     long	size = depth * out->indent + 1;
 
+    if (out->omit_nil && Qnil == value) {
+	return ST_CONTINUE;
+    }
     if (out->end - out->cur <= size) {
 	grow(out, size);
     }
@@ -889,11 +965,11 @@ hash_cb_object(VALUE key, VALUE value, Out out) {
     if (rb_type(key) == T_STRING) {
 	dump_str_obj(key, Qundef, depth, out);
 	*out->cur++ = ':';
-	dump_val(value, depth, out);
+	dump_val(value, depth, out, 0, 0);
     } else if (rb_type(key) == T_SYMBOL) {
 	dump_sym_obj(key, out);
 	*out->cur++ = ':';
-	dump_val(value, depth, out);
+	dump_val(value, depth, out, 0, 0);
     } else {
 	int	d2 = depth + 1;
 	long	s2 = size + out->indent + 1;
@@ -921,13 +997,13 @@ hash_cb_object(VALUE key, VALUE value, Out out) {
 	*out->cur++ = ':';
 	*out->cur++ = '[';
 	fill_indent(out, d2);
-	dump_val(key, d2, out);
+	dump_val(key, d2, out, 0, 0);
 	if (out->end - out->cur <= (long)s2) {
 	    grow(out, s2);
 	}
 	*out->cur++ = ',';
 	fill_indent(out, d2);
-	dump_val(value, d2, out);
+	dump_val(value, d2, out, 0, 0);
 	if (out->end - out->cur <= (long)size) {
 	    grow(out, size);
 	}
@@ -988,26 +1064,26 @@ dump_hash(VALUE obj, VALUE clas, int depth, int mode, Out out) {
 	if (',' == *(out->cur - 1)) {
 	    out->cur--; // backup to overwrite last comma
 	}
-	if (0 == out->opts->dump_opts) {
+	if (!out->opts->dump_opts.use) {
 	    if (out->end - out->cur <= (long)size) {
 		grow(out, size);
 	    }
 	    fill_indent(out, depth);
 	} else {
-	    size = depth * out->opts->dump_opts->indent_size + out->opts->dump_opts->hash_size + 1;
+	    size = depth * out->opts->dump_opts.indent_size + out->opts->dump_opts.hash_size + 1;
 	    if (out->end - out->cur <= (long)size) {
 		grow(out, size);
 	    }
-	    if (0 < out->opts->dump_opts->hash_size) {
-		strcpy(out->cur, out->opts->dump_opts->hash_nl);
-		out->cur += out->opts->dump_opts->hash_size;
+	    if (0 < out->opts->dump_opts.hash_size) {
+		strcpy(out->cur, out->opts->dump_opts.hash_nl);
+		out->cur += out->opts->dump_opts.hash_size;
 	    }
-	    if (0 < out->opts->dump_opts->indent_size) {
+	    if (0 < out->opts->dump_opts.indent_size) {
 		int	i;
 
 		for (i = depth; 0 < i; i--) {
-		    strcpy(out->cur, out->opts->dump_opts->indent);
-		    out->cur += out->opts->dump_opts->indent_size;
+		    strcpy(out->cur, out->opts->dump_opts.indent_str);
+		    out->cur += out->opts->dump_opts.indent_size;
 		}
 	    }
 	}
@@ -1249,7 +1325,12 @@ dump_data_comp(VALUE obj, int depth, Out out) {
 	volatile VALUE	h = rb_funcall(obj, oj_to_hash_id, 0);
  
 	if (T_HASH != rb_type(h)) {
-	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    // It seems that ActiveRecord implemented to_hash so that it returns
+	    // an Array and not a Hash. To get around that any value returned
+	    // will be dumped.
+
+	    //rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    dump_val(h, depth, out, 0, 0);
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
 
@@ -1266,7 +1347,7 @@ dump_data_comp(VALUE obj, int depth, Out out) {
 
 	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	} else {
-	    dump_val(aj, depth, out);
+	    dump_val(aj, depth, out, 0, 0);
 	}
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
@@ -1349,16 +1430,22 @@ dump_data_obj(VALUE obj, int depth, Out out) {
 }
 
 static void
-dump_obj_comp(VALUE obj, int depth, Out out) {
+dump_obj_comp(VALUE obj, int depth, Out out, int argc, VALUE *argv) {
     if (rb_respond_to(obj, oj_to_hash_id)) {
 	volatile VALUE	h = rb_funcall(obj, oj_to_hash_id, 0);
- 
+
 	if (T_HASH != rb_type(h)) {
-	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    // It seems that ActiveRecord implemented to_hash so that it returns
+	    // an Array and not a Hash. To get around that any value returned
+	    // will be dumped.
+
+	    //rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    dump_val(h, depth, out, 0, 0);
+	} else {
+	    dump_hash(h, Qundef, depth, out->opts->mode, out);
 	}
-	dump_hash(h, Qundef, depth, out->opts->mode, out);
     } else if (rb_respond_to(obj, oj_as_json_id)) {
-	volatile VALUE	aj = rb_funcall(obj, oj_as_json_id, 0);
+	volatile VALUE	aj = rb_funcall2(obj, oj_as_json_id, argc, argv);
 
 	// Catch the obvious brain damaged recursive dumping.
 	if (aj == obj) {
@@ -1366,7 +1453,7 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 
 	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	} else {
-	    dump_val(aj, depth, out);
+	    dump_val(aj, depth, out, 0, 0);
 	}
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
@@ -1456,6 +1543,9 @@ dump_attr_cb(ID key, VALUE value, Out out) {
     size_t	size = depth * out->indent + 1;
     const char	*attr = rb_id2name(key);
 
+    if (out->omit_nil && Qnil == value) {
+	return ST_CONTINUE;
+    }
 #if HAS_EXCEPTION_MAGIC
     if (0 == strcmp("bt", attr) || 0 == strcmp("mesg", attr)) {
 	return ST_CONTINUE;
@@ -1477,7 +1567,7 @@ dump_attr_cb(ID key, VALUE value, Out out) {
 	dump_cstr(buf, strlen(buf), 0, 0, out);
     }
     *out->cur++ = ':';
-    dump_val(value, depth, out);
+    dump_val(value, depth, out, 0, 0);
     out->depth = depth;
     *out->cur++ = ',';
     
@@ -1603,6 +1693,8 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 #else
 	size = d2 * out->indent + 1;
 	for (i = cnt; 0 < i; i--, np++) {
+	    VALUE	value;
+	    
 	    vid = rb_to_id(*np);
 	    attr = rb_id2name(vid);
 #ifdef RUBINIUS_RUBY
@@ -1610,6 +1702,10 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 		continue;
 	    }
 #endif
+	    value = rb_ivar_get(obj, vid);
+	    if (out->omit_nil && Qnil == value) {
+		continue;
+	    }
 	    if (first) {
 		first = 0;
 	    } else {
@@ -1631,7 +1727,7 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 		dump_cstr(buf, strlen(attr) + 1, 0, 0, out);
 	    }
 	    *out->cur++ = ':';
-	    dump_val(rb_ivar_get(obj, vid), d2, out);
+	    dump_val(value, d2, out, 0, 0);
 	    if (out->end - out->cur <= 2) {
 		grow(out, 2);
 	    }
@@ -1652,7 +1748,7 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 	    dump_cstr("~mesg", 5, 0, 0, out);
 	    *out->cur++ = ':';
 	    rv = rb_funcall2(obj, rb_intern("message"), 0, 0);
-	    dump_val(rv, d2, out);
+	    dump_val(rv, d2, out, 0, 0);
 	    if (out->end - out->cur <= 2) {
 		grow(out, 2);
 	    }
@@ -1665,7 +1761,7 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 	    dump_cstr("~bt", 3, 0, 0, out);
 	    *out->cur++ = ':';
 	    rv = rb_funcall2(obj, rb_intern("backtrace"), 0, 0);
-	    dump_val(rv, d2, out);
+	    dump_val(rv, d2, out, 0, 0);
 	    if (out->end - out->cur <= 2) {
 		grow(out, 2);
 	    }
@@ -1684,7 +1780,12 @@ dump_struct_comp(VALUE obj, int depth, Out out) {
 	volatile VALUE	h = rb_funcall(obj, oj_to_hash_id, 0);
  
 	if (T_HASH != rb_type(h)) {
-	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    // It seems that ActiveRecord implemented to_hash so that it returns
+	    // an Array and not a Hash. To get around that any value returned
+	    // will be dumped.
+
+	    //rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
+	    dump_val(h, depth, out, 0, 0);
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
     } else if (rb_respond_to(obj, oj_as_json_id)) {
@@ -1696,7 +1797,7 @@ dump_struct_comp(VALUE obj, int depth, Out out) {
 
 	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	} else {
-	    dump_val(aj, depth, out);
+	    dump_val(aj, depth, out, 0, 0);
 	}
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs = rb_funcall(obj, oj_to_json_id, 0);
@@ -1771,14 +1872,17 @@ dump_struct_obj(VALUE obj, int depth, Out out) {
     size = d3 * out->indent + 2;
 #ifdef RSTRUCT_LEN
     {
-	VALUE	*vp;
+	const VALUE	*vp;
 
-	for (i = (int)RSTRUCT_LEN(obj), vp = RSTRUCT_PTR(obj); 0 < i; i--, vp++) {
+#ifndef RSTRUCT_CONST_PTR
+# define RSTRUCT_CONST_PTR(st) (const VALUE *)RSTRUCT_PTR(st)
+#endif
+	for (i = (int)RSTRUCT_LEN(obj), vp = RSTRUCT_CONST_PTR(obj); 0 < i; i--, vp++) {
 	    if (out->end - out->cur <= (long)size) {
 		grow(out, size);
 	    }
 	    fill_indent(out, d3);
-	    dump_val(*vp, d3, out);
+	    dump_val(*vp, d3, out, 0, 0);
 	    *out->cur++ = ',';
 	}
     }
@@ -1793,7 +1897,7 @@ dump_struct_obj(VALUE obj, int depth, Out out) {
 		grow(out, size);
 	    }
 	    fill_indent(out, d3);
-	    dump_val(rb_struct_aref(obj, INT2FIX(i)), d3, out);
+	    dump_val(rb_struct_aref(obj, INT2FIX(i)), d3, out, 0, 0);
 	    *out->cur++ = ',';
 	}
     }
@@ -1834,55 +1938,81 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 	dump_cstr(class_name, clen, 0, 0, out);
 	*out->cur++ = ',';
     }
-    size = d2 * out->indent + 1;
-    for (idp = odd->attrs, fp = odd->attrFuncs; 0 != *idp; idp++, fp++) {
-	size_t	nlen;
+    if (odd->raw) {
+	v = rb_funcall(obj, *odd->attrs, 0);
+	if (Qundef == v || T_STRING != rb_type(v)) {
+	    rb_raise(rb_eEncodingError, "Invalid type for raw JSON.\n");
+	} else {	    
+	    const char	*s = rb_string_value_ptr((VALUE*)&v);
+	    int		len = RSTRING_LEN(v);
+	    const char	*name = rb_id2name(*odd->attrs);
+	    size_t	nlen = strlen(name);
 
-	if (out->end - out->cur <= (long)size) {
-	    grow(out, size);
-	}
-	name = rb_id2name(*idp);
-	nlen = strlen(name);
-	if (0 != *fp) {
-	    v = (*fp)(obj);
-	} else if (0 == strchr(name, '.')) {
-	    v = rb_funcall(obj, *idp, 0);
-	} else {
-	    char	nbuf[256];
-	    char	*n2 = nbuf;
-	    char	*n;
-	    char	*end;
-	    ID		i;
-	    
-	    if (sizeof(nbuf) <= nlen) {
-		n2 = strdup(name);
-	    } else {
-		strcpy(n2, name);
+	    size = len + d2 * out->indent + nlen + 10;
+	    if (out->end - out->cur <= (long)size) {
+		grow(out, size);
 	    }
-	    n = n2;
-	    v = obj;
-	    while (0 != (end = strchr(n, '.'))) {
-		*end = '\0';
+	    fill_indent(out, d2);
+	    *out->cur++ = '"';
+	    memcpy(out->cur, name, nlen);
+	    out->cur += nlen;
+	    *out->cur++ = '"';
+	    *out->cur++ = ':';
+	    memcpy(out->cur, s, len);
+	    out->cur += len;
+	    *out->cur = '\0';
+	}
+    } else {
+	size = d2 * out->indent + 1;
+	for (idp = odd->attrs, fp = odd->attrFuncs; 0 != *idp; idp++, fp++) {
+	    size_t	nlen;
+
+	    if (out->end - out->cur <= (long)size) {
+		grow(out, size);
+	    }
+	    name = rb_id2name(*idp);
+	    nlen = strlen(name);
+	    if (0 != *fp) {
+		v = (*fp)(obj);
+	    } else if (0 == strchr(name, '.')) {
+		v = rb_funcall(obj, *idp, 0);
+	    } else {
+		char	nbuf[256];
+		char	*n2 = nbuf;
+		char	*n;
+		char	*end;
+		ID		i;
+	    
+		if (sizeof(nbuf) <= nlen) {
+		    n2 = strdup(name);
+		} else {
+		    strcpy(n2, name);
+		}
+		n = n2;
+		v = obj;
+		while (0 != (end = strchr(n, '.'))) {
+		    *end = '\0';
+		    i = rb_intern(n);
+		    v = rb_funcall(v, i, 0);
+		    n = end + 1;
+		}
 		i = rb_intern(n);
 		v = rb_funcall(v, i, 0);
-		n = end + 1;
+		if (nbuf != n2) {
+		    free(n2);
+		}
 	    }
-	    i = rb_intern(n);
-	    v = rb_funcall(v, i, 0);
-	    if (nbuf != n2) {
-		free(n2);
+	    fill_indent(out, d2);
+	    dump_cstr(name, nlen, 0, 0, out);
+	    *out->cur++ = ':';
+	    dump_val(v, d2, out, 0, 0);
+	    if (out->end - out->cur <= 2) {
+		grow(out, 2);
 	    }
+	    *out->cur++ = ',';
 	}
-	fill_indent(out, d2);
-	dump_cstr(name, nlen, 0, 0, out);
-	*out->cur++ = ':';
-	dump_val(v, d2, out);
-	if (out->end - out->cur <= 2) {
-	    grow(out, 2);
-	}
-	*out->cur++ = ',';
+	out->cur--;
     }
-    out->cur--;
     *out->cur++ = '}';
     *out->cur = '\0';
 }
@@ -1893,12 +2023,15 @@ raise_strict(VALUE obj) {
 }
 
 static void
-dump_val(VALUE obj, int depth, Out out) {
+dump_val(VALUE obj, int depth, Out out, int argc, VALUE *argv) {
     int	type = rb_type(obj);
 
     if (MAX_DEPTH < depth) {
 	rb_raise(rb_eNoMemError, "Too deeply nested.\n");
     }
+#ifdef OJ_DEBUG
+    printf("Oj-debug: dump_val %s\n", rb_class2name(rb_obj_class(obj)));
+#endif
     switch (type) {
     case T_NIL:		dump_nil(out);				break;
     case T_TRUE:	dump_true(out);				break;
@@ -1965,7 +2098,7 @@ dump_val(VALUE obj, int depth, Out out) {
 		switch (out->opts->mode) {
 		case StrictMode:	dump_data_strict(obj, out);	break;
 		case NullMode:		dump_data_null(obj, out);	break;
-		case CompatMode:	dump_obj_comp(obj, depth, out);	break;
+		case CompatMode:	dump_obj_comp(obj, depth, out, argc, argv);	break;
 		case ObjectMode:
 		default:		dump_obj_obj(obj, depth, out);	break;
 		}
@@ -1988,12 +2121,24 @@ dump_val(VALUE obj, int depth, Out out) {
 		case NullMode:		dump_nil(out);			break;
 		case CompatMode:
 		case ObjectMode:
-		default:		dump_obj_comp(obj, depth, out);	break;
+		default:		dump_obj_comp(obj, depth, out, argc, argv);	break;
 		}
 		break;
 	    default:
-		rb_raise(rb_eNotImpError, "Failed to dump '%s' Object (%02x)\n",
-			 rb_class2name(rb_obj_class(obj)), rb_type(obj));
+		switch (out->opts->mode) {
+		case StrictMode:	raise_strict(obj);		break;
+		case NullMode:		dump_nil(out);			break;
+		case CompatMode: {
+		    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+		    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
+		    break;
+		}
+		case ObjectMode:
+		default:
+		    rb_raise(rb_eNotImpError, "Failed to dump '%s' Object (%02x)\n",
+			     rb_class2name(rb_obj_class(obj)), rb_type(obj));
+		    break;
+		}
 		break;
 	    }
 	}
@@ -2002,6 +2147,11 @@ dump_val(VALUE obj, int depth, Out out) {
 
 void
 oj_dump_obj_to_json(VALUE obj, Options copts, Out out) {
+    oj_dump_obj_to_json_using_params(obj, copts, out, 0, 0);
+}
+
+void
+oj_dump_obj_to_json_using_params(VALUE obj, Options copts, Out out, int argc, VALUE *argv) {
     if (0 == out->buf) {
 	out->buf = ALLOC_N(char, 4096);
 	out->end = out->buf + 4095 - BUFFER_EXTRA; // 1 less than end plus extra for possible errors
@@ -2015,7 +2165,7 @@ oj_dump_obj_to_json(VALUE obj, Options copts, Out out) {
 	oj_cache8_new(&out->circ_cache);
     }
     out->indent = copts->indent;
-    dump_val(obj, 0, out);
+    dump_val(obj, 0, out, argc, argv);
     if (0 < out->indent) {
 	switch (*(out->cur - 1)) {
 	case ']':
@@ -2043,6 +2193,7 @@ oj_write_obj_to_file(VALUE obj, const char *path, Options copts) {
     out.buf = buf;
     out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
+    out.omit_nil = copts->dump_opts.omit_nil;
     oj_dump_obj_to_json(obj, copts, &out);
     size = out.cur - out.buf;
     if (0 == (f = fopen(path, "w"))) {
@@ -2078,6 +2229,7 @@ oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts) {
     out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
     oj_dump_obj_to_json(obj, copts, &out);
+    out.omit_nil = copts->dump_opts.omit_nil;
     size = out.cur - out.buf;
     if (oj_stringio_class == clas) {
 	rb_funcall(stream, oj_write_id, 1, rb_str_new(out.buf, size));
@@ -2306,6 +2458,7 @@ oj_write_leaf_to_file(Leaf leaf, const char *path, Options copts) {
     out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
     oj_dump_leaf_to_json(leaf, copts, &out);
+    out.omit_nil = copts->dump_opts.omit_nil;
     size = out.cur - out.buf;
     if (0 == (f = fopen(path, "w"))) {
 	rb_raise(rb_eIOError, "%s\n", strerror(errno));
@@ -2462,7 +2615,7 @@ oj_str_writer_push_value(StrWriter sw, VALUE val, const char *key) {
 	    *sw->out.cur++ = ':';
 	}
     }
-    dump_val(val, sw->depth, &sw->out);
+    dump_val(val, sw->depth, &sw->out, 0, 0);
 }
 
 void
