@@ -106,37 +106,20 @@ class Juice < Minitest::Test
                    :nilnil=>false,
                    :allow_gc=>true,
                    :quirks_mode=>true,
+                   :allow_invalid_unicode=>false,
                    :float_precision=>15,
                    :mode=>:object,
                    :escape_mode=>:json,
-                   :time_format=>:unix,
+                   :time_format=>:unix_zone,
                    :bigdecimal_load=>:auto,
                    :create_id=>'json_class'}, opts)
   end
 =end
-
   def test_set_options
-    orig ={
-      :indent=>0,
-      :second_precision=>9,
-      :circular=>false,
-      :class_cache=>true,
-      :auto_define=>false,
-      :symbol_keys=>false,
-      :bigdecimal_as_decimal=>true,
-      :use_to_json=>true,
-      :nilnil=>false,
-      :allow_gc=>true,
-      :quirks_mode=>true,
-      :float_precision=>15,
-      :mode=>:object,
-      :escape_mode=>:json,
-      :time_format=>:unix,
-      :bigdecimal_load=>:auto,
-      :create_id=>'json_class'}
-    o2 = {
-      :indent=>4,
-      :second_precision=>7,
+    orig = Oj.default_options()
+    alt ={
+      :indent=>" - ",
+      :second_precision=>5,
       :circular=>true,
       :class_cache=>false,
       :auto_define=>true,
@@ -145,19 +128,30 @@ class Juice < Minitest::Test
       :use_to_json=>false,
       :nilnil=>true,
       :allow_gc=>false,
-      :quirks_mode=>true,
-      :float_precision=>15,
-      :mode=>:compat,
-      :escape_mode=>:json,
-      :time_format=>:ruby,
-      :bigdecimal_load=>:bigdecimal,
-      :create_id=>nil}
-    o3 = { :indent => 4 }
-    Oj.default_options = o2
+      :quirks_mode=>false,
+      :allow_invalid_unicode=>true,
+      :float_precision=>13,
+      :mode=>:strict,
+      :escape_mode=>:ascii,
+      :time_format=>:unix_zone,
+      :bigdecimal_load=>:float,
+      :create_id=>'classy',
+      :space=>'z',
+      :array_nl=>'a',
+      :object_nl=>'o',
+      :space_before=>'b',
+      :nan=>:huge,
+      :hash_class=>Hash,
+      :omit_nil=>false,
+    }
+    Oj.default_options = alt
     opts = Oj.default_options()
-    assert_equal(o2, opts);
-    Oj.default_options = o3 # see if it throws an exception
+    assert_equal(alt, opts);
+
     Oj.default_options = orig # return to original
+    # verify back to original
+    opts = Oj.default_options()
+    assert_equal(orig, opts);
   end
 
   def test_nil
@@ -179,6 +173,27 @@ class Juice < Minitest::Test
     dump_and_load(1, false)
   end
 
+  def test_float_parse
+    Oj.default_options = { :float_precision => 16, :bigdecimal_load => :auto }
+=begin
+    n = Oj.load('0.00001234567890123456')
+    assert_equal(Float, n.class)
+    assert_equal('1.234567890123456e-05', "%0.15e" % [n])
+
+    n = Oj.load('-0.00001234567890123456')
+    assert_equal(Float, n.class)
+    assert_equal('-1.234567890123456e-05', "%0.15e" % [n])
+=end
+    n = Oj.load('1000.0000123456789')
+    assert_equal(BigDecimal, n.class)
+    assert_equal('0.10000000123456789E4', n.to_s)
+=begin
+    n = Oj.load('-0.000012345678901234567')
+    assert_equal(BigDecimal, n.class)
+    assert_equal('-0.12345678901234567E-4', n.to_s)
+=end
+  end
+
   def test_float_dump
     Oj.default_options = { :float_precision => 16 }
     assert_equal('1405460727.723866', Oj.dump(1405460727.723866))
@@ -206,6 +221,24 @@ class Juice < Minitest::Test
     assert_equal('-95.640172', Oj.dump(-95.640172))
   end
 
+  def test_nan_dump
+    assert_equal('null', Oj.dump(0/0.0, :mode => :strict, :nan => :null))
+    assert_equal('NaN', Oj.dump(0/0.0, :mode => :strict, :nan => :word))
+    assert_equal('3.3e14159265358979323846', Oj.dump(0/0.0, :mode => :strict, :nan => :huge))
+  end
+
+  def test_infinity_dump
+    assert_equal('null', Oj.dump(1/0.0, :mode => :strict, :nan => :null))
+    assert_equal('Infinity', Oj.dump(1/0.0, :mode => :strict, :nan => :word))
+    assert_equal('3.0e14159265358979323846', Oj.dump(1/0.0, :mode => :strict, :nan => :huge))
+  end
+
+  def test_neg_infinity_dump
+    assert_equal('null', Oj.dump(-1/0.0, :mode => :strict, :nan => :null))
+    assert_equal('-Infinity', Oj.dump(-1/0.0, :mode => :strict, :nan => :word))
+    assert_equal('-3.0e14159265358979323846', Oj.dump(-1/0.0, :mode => :strict, :nan => :huge))
+  end
+
   def test_float
     mode = Oj.default_options()[:mode]
     Oj.default_options = {:mode => :object}
@@ -221,7 +254,7 @@ class Juice < Minitest::Test
     dump_and_load(1/0.0, false)
     # NaN does not always == NaN
     json = Oj.dump(0/0.0)
-    assert_equal('NaN', json)
+    assert_equal('3.3e14159265358979323846', json)
     loaded = Oj.load(json);
     assert_equal(true, loaded.nan?)
     Oj.default_options = {:mode => mode}
@@ -261,6 +294,51 @@ class Juice < Minitest::Test
     assert_equal(json, json2)
   end
 
+  def test_invalid_unicode_raise
+    # validate that an invalid unicode raises unless the :allow_invalid_unicode is true
+    json = %{"x\\ud83dy"}
+    begin
+      obj = Oj.load(json)
+    rescue Exception
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
+  def test_invalid_unicode_ok
+    # validate that an invalid unicode raises unless the :allow_invalid_unicode is true
+    json = %{"x\\ud83dy"}
+    obj = Oj.load(json, :allow_invalid_unicode => true)
+    # The same as what ruby would do with the invalid encoding.
+    assert_equal("x\xED\xA0\xBDy", obj.to_s)
+  end
+
+  def test_dump_options
+    json = Oj.dump({ 'a' => 1, 'b' => [true, false]},
+                   :mode => :compat,
+                   :indent => "--",
+                   :array_nl => "\n",
+                   :object_nl => "#\n",
+                   :space => "*",
+                   :space_before => "~")
+    assert(%{{#
+--"a"~:*1,#
+--"b"~:*[
+----true,
+----false
+--]#
+}} == json ||
+%{{#
+--"b"~:*[
+----true,
+----false
+--],#
+--"a"~:*1#
+}} == json)
+
+  end
+
   def test_null_char
     assert_raises(Oj::ParseError) { Oj.load("\"\0\"") }
     assert_raises(Oj::ParseError) { Oj.load("\"\\\0\"") }
@@ -299,13 +377,7 @@ class Juice < Minitest::Test
 [1,2][3,4]
 {"b":2}
 }
-    begin
-      Oj.load(json)
-    rescue Exception
-      assert(true)
-      return
-    end
-    assert(false, "*** expected an exception")
+    assert_raises(Oj::ParseError) { Oj.load(json) }
   end
 
   # encoding tests
@@ -907,8 +979,10 @@ class Juice < Minitest::Test
     b = Oj.load(json, :mode => :strict)
     assert_equal(30226801971775055948247051683954096612865741943, b)
   end
+
   def test_bignum_object
     dump_and_load(7 ** 55, false)
+    dump_and_load(10 ** 19, false)
   end
 
   # BigDecimal
@@ -1184,7 +1258,8 @@ class Juice < Minitest::Test
   end
 
   def test_comment_c
-    json = %{{
+    json = %{/*before*/
+{
   "x"/*one*/:/*two*/true,
   "y":58,
   "z": [1,2,3]}
@@ -1221,9 +1296,55 @@ class Juice < Minitest::Test
     assert_equal(nil, obj)
   end
 
-  def test_quirks_mode
+  def test_quirks_null_mode
     assert_raises(Oj::ParseError) { Oj.load("null", :quirks_mode => false) }
-    assert_equal nil, Oj.load("null", :quirks_mode => true)
+    assert_equal(nil, Oj.load("null", :quirks_mode => true))
+  end
+
+  def test_quirks_bool_mode
+    assert_raises(Oj::ParseError) { Oj.load("true", :quirks_mode => false) }
+    assert_equal(true, Oj.load("true", :quirks_mode => true))
+  end
+
+  def test_quirks_number_mode
+    assert_raises(Oj::ParseError) { Oj.load("123", :quirks_mode => false) }
+    assert_equal(123, Oj.load("123", :quirks_mode => true))
+  end
+
+  def test_quirks_decimal_mode
+    assert_raises(Oj::ParseError) { Oj.load("123.45", :quirks_mode => false) }
+    assert_equal(123.45, Oj.load("123.45", :quirks_mode => true))
+  end
+
+  def test_quirks_string_mode
+    assert_raises(Oj::ParseError) { Oj.load('"string"', :quirks_mode => false) }
+    assert_equal('string', Oj.load('"string"', :quirks_mode => true))
+  end
+
+  def test_quirks_array_mode
+    assert_equal([], Oj.load("[]", :quirks_mode => false))
+    assert_equal([], Oj.load("[]", :quirks_mode => true))
+  end
+
+  def test_quirks_object_mode
+    assert_equal({}, Oj.load("{}", :quirks_mode => false))
+    assert_equal({}, Oj.load("{}", :quirks_mode => true))
+  end
+
+  def test_omit_nil
+    jam = Jam.new({'a' => 1, 'b' => nil }, nil)
+
+    json = Oj.dump(jam, :omit_nil => true, :mode => :object)
+    assert_equal(%|{"^o":"Juice::Jam","x":{"a":1}}|, json)
+
+    json = Oj.dump(jam, :omit_nil => true, :mode => :compat)
+    assert_equal(%|{"x":{"a":1}}|, json)
+
+    json = Oj.dump({'x' => {'a' => 1, 'b' => nil }, 'y' => nil}, :omit_nil => true, :mode => :strict)
+    assert_equal(%|{"x":{"a":1}}|, json)
+
+    json = Oj.dump({'x' => {'a' => 1, 'b' => nil }, 'y' => nil}, :omit_nil => true, :mode => :null)
+    assert_equal(%|{"x":{"a":1}}|, json)
   end
 
   def dump_and_load(obj, trace=false)
